/**
  *  \file planetscentral/dbclient/dblist.cc
  */

#include "planetscentral/dbclient/dblist.h"
#include "planetscentral/dbclient/dbclient.h"

DbList::DbList(DbClient& client, string_t key)
    : DbKey(client, key)
{ }

int32_t
DbList::size()
{
    return client.callInt(DbRequest().withString("LLEN").withString(key));
}

bool
DbList::empty()
{
    return size() == 0;
}

void
DbList::trimToRange(int32_t start, int32_t end)
{
    client.callVoid(DbRequest().withString("LTRIM").withString(key).withInt(start).withInt(end));
}

DbSort
DbList::sort()
{
    return DbSort(getClient(), getName());
}

/******************************* DbIntList *******************************/

DbIntList::DbIntList(DbClient& client, string_t key)
    : DbList(client, key)
{ }

int32_t
DbIntList::operator[](int32_t index)
{
    return client.callInt(DbRequest().withString("LINDEX").withString(key).withInt(index));
}

int32_t
DbIntList::popFront()
{
    return client.callInt(DbRequest().withString("LPOP").withString(key));
}

int32_t
DbIntList::pushFront(int32_t value)
{
    return client.callInt(DbRequest().withString("LPUSH").withString(key).withInt(value));
}

int32_t
DbIntList::popBack()
{
    return client.callInt(DbRequest().withString("RPOP").withString(key));
}

int32_t
DbIntList::pushBack(int32_t value)
{
    return client.callInt(DbRequest().withString("RPUSH").withString(key).withInt(value));
}

void
DbIntList::set(int32_t index, int32_t value)
{
    client.callVoid(DbRequest().withString("LSET").withString(key).withInt(index).withInt(value));
}

int32_t
DbIntList::removeValue(int32_t value, int32_t count)
{
    return client.callInt(DbRequest().withString("LREM").withString(key).withInt(count).withInt(value));
}

int32_t
DbIntList::transferElement(DbIntList& other)
{
    return client.callInt(DbRequest().withString("RPOPLPUSH").withString(key).withString(other.key));
}

void
DbIntList::getAll(IntList_t& list)
{
    std::auto_ptr<IntValue> val(client.call(DbRequest().withString("LRANGE").withString(key).withInt(0).withInt(-1)));
    toIntListResult(val.get(), list);
}

/****************************** DbStringList *****************************/

DbStringList::DbStringList(DbClient& client, string_t key)
    : DbList(client, key)
{ }

string_t
DbStringList::operator[](int32_t index)
{
    return client.callString(DbRequest().withString("LINDEX").withString(key).withInt(index));
}

string_t
DbStringList::popFront()
{
    return client.callString(DbRequest().withString("LPOP").withString(key));
}

string_t
DbStringList::pushFront(string_t value)
{
    return client.callString(DbRequest().withString("LPUSH").withString(key).withString(value));
}

string_t
DbStringList::popBack()
{
    return client.callString(DbRequest().withString("RPOP").withString(key));
}

string_t
DbStringList::pushBack(string_t value)
{
    return client.callString(DbRequest().withString("RPUSH").withString(key).withString(value));
}

void
DbStringList::set(int32_t index, string_t value)
{
    client.callVoid(DbRequest().withString("LSET").withString(key).withInt(index).withString(value));
}

int32_t
DbStringList::removeValue(string_t value, int32_t count)
{
    return client.callInt(DbRequest().withString("LREM").withString(key).withInt(count).withString(value));
}

string_t
DbStringList::transferElement(DbStringList& other)
{
    return client.callString(DbRequest().withString("RPOPLPUSH").withString(key).withString(other.key));
}

void
DbStringList::getAll(StringList_t& list)
{
    std::auto_ptr<IntValue> val(client.call(DbRequest().withString("LRANGE").withString(key).withInt(0).withInt(-1)));
    toStringListResult(val.get(), list);
}
