/**
  *  \file planetscentral/talk/userpm.h
  *  \brief Talk: User Message
  */
#ifndef C2SERVER_PLANETSCENTRAL_TALK_USERPM_H
#define C2SERVER_PLANETSCENTRAL_TALK_USERPM_H

#include "planetscentral/dbclient/dbsubtree.h"
#include "cpluslib/types.h"
#include "cpluslib/string.h"

/** A user's personal message.
    This uses a single-instance store for messages. Each message has a
    reference counter. A message to multiple users is stored only once.
    Also, the sender's outbox copy is another reference of the message.

    Individual messages have no access control. In the protocol,
    messages are always addressed using a user folder Id (ufid) and a
    PM Id. Since folder addresses are user-specific, this will always
    address the user's copy, and a message a user shall not be able to
    access will just not be addressable.

    As a drawback, it is not possible to make a stable URL for a message
    because that would always have to include the folder. */
class UserPM {
 public:
    UserPM(DbClient& client, int32_t pmId);

    DbHash            header();
    DbHashStringField author();
    DbHashStringField receivers();
    DbHashIntField    time();
    DbHashStringField subject();
    DbHashIntField    referenceCounter();

    DbHashIntField    flags(string_t forUser);

    DbString          text();

    IntValue*         describe(string_t forUser);

    int32_t           getId() const;

    void addReference();
    void removeReference();

    static int32_t allocatePM(DbClient& client);

 private:
    DbSubtree pm;
    const int32_t pmId;
};

#endif
